/**********************************************************************

Poti.cpp
Reads 2 pots and 4 switches/pushbuttons for loca control
no computer necessary. Push button on A5 turns on power to main track only
Uses channel 10 and 11, so only channels 1...9 can be used by JMRI etc. 
Part of DCC++ BASE STATION for the Arduino

**********************************************************************/


#include "Poti.h"
#include "DCCpp_Uno.h"
#include "Arduino.h"
//#include "Accessories.h"
//#include "Sensor.h"
//#include "Outputs.h"
//#include "EEStore.h"
//#include "Comm.h"

//extern int __heap_start, *__brkval;

///////////////////////////////////////////////////////////////////////////////

char Poti::commandString[MAX_COMMAND_LENGTH+1];
volatile RegisterList *Poti::mRegs;
int Poti::OldCabSpeed[]= {0,0};
bool Poti::flipflop = false;
bool Poti::PotiActive = false;
unsigned char Poti::OldFunctions = 0;


///////////////////////////////////////////////////////////////////////////////

void Poti::init(volatile RegisterList *_mRegs){
  mRegs=_mRegs;
  sprintf(commandString,"");
} // Poti:Poti

///////////////////////////////////////////////////////////////////////////////
// A5 with a push button has several functions:
// When pressed during startup the system shows the configuration as defined by the original software
// When pressed after startup it turns on power to the main track (only) and activates the local controls:
// - 2 Potentiometers for address 3 and 6 and 2 functions each (F1/F2). In parallel 10 additional locos 
//   can be run with a computer hooked up 
// If not pressed at all, the system behaves like the original DCC++ software with 12 loco cannels
void Poti::process(){
  
  //Md Poti
    int sensorValue;
    int CabSpeed;   //map speed to 0..264 because of 10 counts dead band
    int forward;
    int OutFunction;
    char command[20];
    unsigned char NewFunctions;

	if (!PotiActive){                             // Check if local controls enabled
    if(!digitalRead(A5)){
     digitalWrite(SIGNAL_ENABLE_PIN_MAIN,HIGH);     // Reset main power to "on"
	   PotiActive= true;
    }
	} else {
    if (flipflop){
     sensorValue = analogRead(A2);
     CabSpeed = map (sensorValue,0,1023,0,262);   //map speed to 0..264 because of 10 counts dead band

     if ((CabSpeed > 126)&&(CabSpeed < 137)){  // within dead band set speed to mid (0)   
      CabSpeed = 126;
     }
//  Serial.println(CabSpeed);
     if (CabSpeed > 136){
       CabSpeed = CabSpeed - 136;
        forward = 1;
     }
    else{
      CabSpeed = 126 - CabSpeed;
      forward = 0;
   } 
  // print out the value you read:
    if (1==CabSpeed)CabSpeed = 0;                         // Avoid emergency stop
    if (CabSpeed!=OldCabSpeed[0])
    {
      OldCabSpeed[0] = CabSpeed;
      sprintf(command,"10 FIRST_LOCO_ADDRESS %i %i",CabSpeed,forward);     //Channel 0 to 9 used by normal throttles
      mRegs->setThrottle(command);
//      sprintf(command,"6 128");
//      mRegs->setFunction(command);
    }
    }
    else {
     sensorValue = analogRead(A3);
     CabSpeed = map (sensorValue,0,1023,0,262);   //map speed to 0..264 because of 10 counts dead band

     if ((CabSpeed > 126)&&(CabSpeed < 137)){  // within dead band set speed to mid (0)   
      CabSpeed = 126;
     }
//  Serial.println(CabSpeed);
    if (CabSpeed > 136){
      CabSpeed = CabSpeed - 136;
      forward = 1;
    }
    else{
      CabSpeed = 126 - CabSpeed;
      forward = 0;
   } 

    if (1==CabSpeed)CabSpeed = 0;                         // Avoid emergency stop
    if (CabSpeed!=OldCabSpeed[1])
     {
       OldCabSpeed[1] = CabSpeed;
        sprintf(command,"11 SECOND_LOCO_ADDRESS %i %i",CabSpeed,forward);     //Channel 0 to 9 used by normal throttles
        mRegs->setThrottle(command);
 
     }
    }// here if EMO, rest as else EMO reset when all throttles  0 LED as PWM to show zeros

// Functions, only if any change
//  OutFunction:  128 + F1*1 + F2*2 + F3*4 + F4*8 + F0*16
    OutFunction = 128;
 
    NewFunctions = PIND & B11010100;                      // read PortD and mask only 4 function pins

    if (NewFunctions != OldFunctions){
      if ((NewFunctions ^ OldFunctions) < 32) {           // Is the change in bits 6,7 or 2,4 

      if ((NewFunctions & B00000100)!=0){                 // Bit 2 function 2     
        OutFunction = OutFunction + 2;
        }

      if ((NewFunctions & B00010000)!=0){                 // Bit 4 function 3
        OutFunction = OutFunction + 4;
        } 
      sprintf(command,"FIRST_LOCO_ADDRESS %i",OutFunction);
      mRegs->setFunction(command);
      } 
      else {
      OutFunction = 128;
      if ((NewFunctions & B01000000)!=0){                 // function 2 and 3 second channel     
        OutFunction = OutFunction + 2;
        }

      if ((NewFunctions & B10000000)!=0){
        OutFunction = OutFunction + 4;
        } 
      sprintf(command,"SECOND_LOCO_ADDRESS %i",OutFunction);
      mRegs->setFunction(command);
      }
      OldFunctions = NewFunctions;
    }

    flipflop = !flipflop;
	}
  
} // Poti::process

///////////////////////////////////////////////////////////////////////////////
